if [[ -z "${ENV}" ]]; then
    echo "must set ENV variable"
    exit 1
fi

##
## Retrieves data-retention files from S3
## These files are generated by data-eng team and contain a list of deregistered organizations
##
get_deregistered_orgs_files() {
    rm -rf /tmp/cleanup-${ENV}
    mkdir -p /tmp/cleanup-${ENV}
    aws s3 cp --recursive s3://${ENV}-data-retention/ /tmp/cleanup-${ENV}/
}

##
## Aggregate the deregistered orgs files into a single list of org IDs
##
build_deregistered_org_keys() {
    if [[ -f "/tmp/${ENV}-deregistered-org-keys" ]]; then
        echo "ERROR - File already exists! /tmp/${ENV}-deregistered-org-keys"
        exit 1
    fi
    for f in $(ls -1 /tmp/cleanup-${ENV}/${ENV}*.csv); do
        cat $f | cut -d',' -f2 | grep -v org_key | grep -v rows >> /tmp/${ENV}-deregistered-org-keys
    done
}

## NOTE: Updated for asset index
## Get counts of devicestats entries for a given org ID
##
count_devicestats_for_org_key() {
  if [[ $string =~ "a_asset" ]]; then
    count=$(curl -s "Content-Type:application/json" -u "${ES_USER}:${ES_PASSWORD}" "https://${ES_HOST}:443/${ES_INDEX}/_count?routing=${1}&q=org_key:${1}" | jq .count)
  else
    count=$(curl -s "Content-Type:application/json" -u "${ES_USER}:${ES_PASSWORD}" "https://${ES_HOST}:443/${ES_INDEX}/_count?q=org_key:${1}" | jq .count)
  fi
  echo "$1 $count" >> /tmp/${ENV}-deregistered-org-keys-counts
}

## NOTE: Updated for asset index
## Cleanup with delete-by-query, but do it in batches to enable slow rollout
## Given a list of org IDs at /tmp/${ENV}-deregistered-org-ids-filtered
## Hint: run update_devicestats_counts first
##
cleanup_by_query() {
    FILE=/tmp/${ENV}-deregistered-org-keys-filtered
    IFS=$'\n'

    ## LOOPS * BATCH = number of orgs to run this for
    ## START allows starting at an offset in a list of org IDs
    ## 100 * 100 = 10000 orgs
    LOOP=1
    LOOPS=100
    START=0
    BATCH=100
    i=$(tail -n +${START} $FILE | head -n ${BATCH})

    while [[ -n $i && ${LOOPS} -ge ${LOOP} ]]; do
        echo "starting at line=${START} on loop=${LOOP}"
        for j in $i; do
            org_key=$(echo $j | cut -d' ' -f1)
            echo "Processing ${org_key}"
#            echo "curl -s -XPOST --header \"Content-Type:application/json\" -u \"${ES_USER}:${ES_PASSWORD}\" \"https://${ES_HOST}:443/a_asset/_delete_by_query?routing=$org_key&conflicts=proceed&q=org_key:$org_key\" | jq"
            if [[ $string =~ "a_asset" ]]; then
              curl -s -XPOST --header "Content-Type:application/json" -u "${ES_USER}:${ES_PASSWORD}" "https://${ES_HOST}:443/${ES_INDEX}/_delete_by_query?routing=$org_key&conflicts=proceed&q=org_key:$org_key" | jq
            else
              curl -s -XPOST --header "Content-Type:application/json" -u "${ES_USER}:${ES_PASSWORD}" "https://${ES_HOST}:443/${ES_INDEX}/_delete_by_query?conflicts=proceed&q=org_key:$org_key" | jq
            fi
        done
        ((START=START+${BATCH}))
        ((LOOP=LOOP+1))
        i=$(tail -n +${START} $FILE | head -n ${BATCH})
    done
}

##
## Given a list of deregistered-org-ids, find the number of entries that org has
## Final list will be in /tmp/${ENV}-deregistered-org-ids-filtered
##
update_devicestats_counts() {
    # 3. sort
    rm -f /tmp/${ENV}-deregistered-org-keys-counts
    while read -r line
    do
      count_devicestats_for_org_key "$line"
    done < /tmp/${ENV}-deregistered-org-keys
    sort -n -k 2 /tmp/${ENV}-deregistered-org-keys-counts > /tmp/${ENV}-deregistered-org-keys-sorted
    grep -v ' 0' /tmp/${ENV}-deregistered-org-keys-sorted | uniq > /tmp/${ENV}-deregistered-org-keys-filtered
    echo "Found $(wc -l  /tmp/${ENV}-deregistered-org-keys-filtered) orgs with devicestats data"
}

##
## Does a find for document IDs, starting at earliest createTime and then a delete on each entry
## Not ideal for production, but sometimes necessary if the document's _version is invalid, preventing a delete_by_query
##
find_and_delete_devicestats_for_org() {
    local max_entries_to_delete=1000
    local index=a_devicestats

    DOC_IDS=$(curl -s -XPOST --header "Content-Type:application/json" --url "$(cbd es)/$index/_search?q=organizationId:$1&size=$max_entries_to_delete&sort=createTime:asc" | jq -r .hits.hits[]._id)
    for d in ${DOC_IDS}; do
        curl -s -XDELETE --header "Content-Type:application/json" --url "$(cbd es)/$index/device/${d}?routing=$1" | jq
    done
}
